//+------------------------------------------------------------------+
//|                                                     ADX Slope.mq4|
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"

#property indicator_separate_window
#property indicator_buffers 5
#property indicator_color1  clrRed
#property indicator_color2  clrLimeGreen
#property indicator_color3  clrLimeGreen
#property indicator_color4  clrOrangeRed
#property indicator_color5  clrOrangeRed
#property indicator_width1  1
#property indicator_width2  3
#property indicator_width3  3
#property indicator_width4  3
#property indicator_width5  3
#property strict

//
//
//
//
//

extern ENUM_TIMEFRAMES TimeFrame       = PERIOD_CURRENT; // Time frame to use
extern int             ADXPeriod       = 14;             // Adx Period
extern bool            alertsOn        = true;           // alerts on?
extern bool            alertsOnCurrent = false;          // alerts on current (still open) bar?
extern bool            alertsMessage   = true;           // alerts message?
extern bool            alertsSound     = false;          // alerts sound?
extern bool            alertsNotify    = false;          // alerts notification?
extern bool            alertsEmail     = false;          // alerts email?
extern string          soundFile       = "alert2.wav";   // alerts sound file
extern bool            Interpolate     = true;           // Interpolate in multi time frame mode?



//
//
//
//
//

double ADX[];
double ADXua[];
double ADXub[];
double ADXda[];
double ADXdb[];
double slope[];
string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,ADX);
   SetIndexBuffer(1,ADXua);
   SetIndexBuffer(2,ADXub);
   SetIndexBuffer(3,ADXda);
   SetIndexBuffer(4,ADXdb);
   SetIndexBuffer(5,slope);
      indicatorFileName = WindowExpertName();
      returnBars        = TimeFrame==-99;
      TimeFrame         = MathMax(TimeFrame,_Period);
   IndicatorShortName(timeFrameToString(TimeFrame)+" ADX slope ("+(string)ADXPeriod+") ");
return(0);
}

//
//
//
//
//

int start()
{ 
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
            if (returnBars) { ADX[0] = limit+1; return(0); }
            if (TimeFrame != _Period)
            {
               limit = (int)MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/_Period));
               if (slope[limit] == 1) CleanPoint(limit,ADXua,ADXub);
               if (slope[limit] ==-1) CleanPoint(limit,ADXda,ADXdb);
               for (int i=limit;i>=0;i--)
               {
                  int y = iBarShift(NULL,TimeFrame,Time[i]);
                     ADX[i]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,ADXPeriod,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,0,y);
                     slope[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,ADXPeriod,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsNotify,alertsEmail,soundFile,5,y);
                     ADXua[i] = EMPTY_VALUE;      
                     ADXub[i] = EMPTY_VALUE;
                     ADXda[i] = EMPTY_VALUE;      
                     ADXdb[i] = EMPTY_VALUE;

                     //
                     //
                     //
                     //
                     //
      
                     if (!Interpolate || (i>0 && y==iBarShift(NULL,TimeFrame,Time[i-1]))) continue;

                     int n,k; datetime time = iTime(NULL,TimeFrame,y);
                        for(n = 1; i+n<Bars && Time[i+n] >= time; n++) continue;	
                        for(k = 1; i+n<Bars && i+k<Bars && k<n; k++) ADX[i+k] = ADX[i] + (ADX[i+n]-ADX[i])*k/n;
               }
               for (int i=limit;i>=0;i--)
               {
                  if (slope[i] == 1) PlotPoint(i,ADXua,ADXub,ADX);
                  if (slope[i] ==-1) PlotPoint(i,ADXda,ADXdb,ADX);
               }
               return(0);
            }               

   //
   //
   //
   //
   //
   
   if (slope[limit] == 1) CleanPoint(limit,ADXua,ADXub);
   if (slope[limit] ==-1) CleanPoint(limit,ADXda,ADXdb);
   for(int i = limit; i >= 0; i--)
   {
         ADX[i]   = iADX(NULL,0,ADXPeriod,PRICE_CLOSE,MODE_MAIN,i);
         ADXua[i] = EMPTY_VALUE;      
         ADXub[i] = EMPTY_VALUE;
         ADXda[i] = EMPTY_VALUE;      
         ADXdb[i] = EMPTY_VALUE;
         if (i<(Bars-1))
         {
           slope[i] = slope[+1];
           if (ADX[i] > ADX[i+1]) slope[i] = 1;
           if (ADX[i] < ADX[i+1]) slope[i] =-1;
           if (slope[i] == 1) PlotPoint(i,ADXua,ADXub,ADX);
           if (slope[i] ==-1) PlotPoint(i,ADXda,ADXdb,ADX);
         } 
   }  
   
   //
   //
   //
   //
   //
   
   if (alertsOn)
   {
      int whichBar = 1; if (alertsOnCurrent) whichBar = 0;
      if (slope[whichBar] != slope[whichBar+1])
      {
         if (slope[whichBar]== 1) doAlert(whichBar,"STRONG UP");
         if (slope[whichBar]==-1) doAlert(whichBar,"WEAK down");
      }
   }
return(0);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(timeFrameToString(_Period)+" "+Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," ADX ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsNotify)  SendNotification(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(), Period(), " ADX "),message);
             if (alertsSound)   PlaySound(soundFile);
      }
}

//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}